<?php
/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

require_once (MC_LIB_PATH.'Plattform.php');

/**
 * Abstract base-class for esrender-modules.
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
abstract class ESRender_Module_Base
extends Plattform
implements ESRender_Module_Interface
{

	/**
	 * @var ESObject
	 */
	protected $_ESOBJECT = null;

	/**
	 * This module's name
	 *
	 * @var string
	 */
	protected $Name = '';

	/**
	 *
	 * @var string
	 */
	protected $render_path = '';

	/**
	 *
	 * @param string $Name
	 * @param ESRender_Application_Interface $RenderApplication
	 * @param ESObject $p_esobject
	 * @param Logger $Logger
	 * @param Phools_Template_Interface $Template
	 */
	public function __construct(
		$Name,
		ESRender_Application_Interface $RenderApplication,
		ESObject $p_esobject,
		Logger $Logger,
		Phools_Template_Interface $Template)
	{
		parent::__construct();

		$this
			->setName($Name)
			->setRenderApplication($RenderApplication)
			->setLogger($Logger)
			->setTemplate($Template);

		$this->_ESOBJECT = $p_esobject;
	}

	public function __destruct()
	{
		$this->_ESOBJECT = null;
		$this->Logger = null;
		$this->Template = null;
		$this->RenderApplication = null;
	}

	/**
	 * Prepare data to be used in templates.
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 *
	 * @return array
	 */
	protected function prepareRenderData(
			EsApplication $RemoteApplication,
			array $requestData)
	{
		$license = '';
		if ( $this->_ESOBJECT->getLicense() )
		{
			
			$license = $this->_ESOBJECT->getLicense()->renderFooter($this->getTemplate());
		}


		$Data = array(
			'title' => $this->_ESOBJECT->getTitle(),
       'license' => $license,
			'width' => $requestData['width'],
			'height' => $requestData['height'],
		);

		return $Data;
	}

	/**
	 * Display current ESObject.
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 */
	abstract protected function display(
		EsApplication $RemoteApplication,
		array $requestData);

	/**
	 * Download current ESObject.
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 */
	abstract protected function download(
		EsApplication $RemoteApplication,
		array $requestData);

	/**
	 * Render inline-portion for current ESObject.
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 */
	abstract protected function inline(
		EsApplication $RemoteApplication,
		array $requestData);

	/**
	 * Helper-method to generate a URL to icon for the object's license.
	 *
	 * @param string $license
	 */
	protected function getLicenseIconUrl($license)
	{
		$url = MC_SCHEME . '://' . MC_HOST . MC_PATH . '/theme/default/license/';
		switch( strtoupper($license) )
		{
			case ESObject::COMMON_LICENSE_CUSTOM:
				$url .= ESObject::COMMON_LICENSE_CUSTOM . '.png';
				break;
			case ESObject::COMMON_LICENSE_EDU_NC:
				$url .= ESObject::COMMON_LICENSE_EDU_NC . '.png';
				break;
			case ESObject::COMMON_LICENSE_EDU_NC_ND:
				$url .= ESObject::COMMON_LICENSE_EDU_NC_ND . '.png';
				break;
			case ESObject::COMMON_LICENSE_EDU_P_NR:
				$url .= ESObject::COMMON_LICENSE_EDU_P_NR . '.png';
				break;
			case ESObject::COMMON_LICENSE_EDU_P_NR_ND:
				$url .= ESObject::COMMON_LICENSE_EDU_P_NR_ND . '.png';
				break;
			case ESObject::CC_BY:
				$url .= 'by.png';
				break;
			case ESObject::CC_BY_NC:
				$url .= 'by-nc.png';
				break;
			case ESObject::CC_BY_NC_ND:
				$url .= 'by-nc-nd.png';
				break;
			case ESObject::CC_BY_NC_SA:
				$url .= 'by-nc-sa.png';
				break;
			case ESObject::CC_BY_ND:
				$url .= 'by-nd.png';
				break;
			case ESObject::CC_BY_SA:
				$url .= 'by-sa.png';
				break;

			default:
				error_log('Unhandled or unknown license.');
				return false;
		}

		return $url;
	}

	/**
	 *
	 */
	protected function _buildUsername(array $requestData)
	{
		$username = $requestData['user_name'];
		$username .= '@' . $requestData['app_id'];

		return $username;
	}

	/**
	 *
	 * @param string $Sql
	 * @param array $requestData
	 * @param string $requiredInstanceVersion
	 */
	protected function refineInstanceConstraints($Sql, array $requestData, $requiredInstanceVersion)
	{
		// nothing to refine by default.
		return $Sql;
	}

	/**
	 * Test if this object already exists for this module. This method
	 * checks only ESRender's ESOBJECT-table to for existance of this
	 * object. Override this method to implement module-specific behaviour
	 * (@see modules/moodle/mod_moodle.php).
	 *
	 * (non-PHPdoc)
	 * @see ESRender_Module_Interface::instanceExists()
	 */
	public function instanceExists(
		ESObject $ESObject,
		array $requestData,
		$requiredInstanceVersion)
	{
		$Logger = $this->getLogger();

		$Sql = 'SELECT *'
			.' FROM '.$this->DB->quoteIdentifier('ESOBJECT')
			.' WHERE '
			.' '.$this->DB->quoteIdentifier('ESOBJECT_REP_ID').' = '.$this->DB->quote($requestData['rep_id'], 'text')
			.' AND '.$this->DB->quoteIdentifier('ESOBJECT_OBJECT_ID').' = '.$this->DB->quote($ESObject->getObjectID(), 'text');

		if ( ! empty($requiredInstanceVersion) )
		{
			$Sql .= ' AND '.$this->DB->quoteIdentifier('ESOBJECT_OBJECT_VERSION').' = '.$this->DB->quote($requiredInstanceVersion, 'text');
		}
		else
		{
			$Sql .= ' AND '.$this->DB->quoteIdentifier('ESOBJECT_OBJECT_VERSION').' IS NULL';
		}

		/*
		 * As every module can have different reuirements, when an already
		 * rendered instance is considered appropriate for current request,
		 * let it refine the search-conditions.
		 */
		$Sql = $this->refineInstanceConstraints($Sql, $requestData, $requiredInstanceVersion);

		switch( $this->DB->getPhptype() )
		{
			case 'mysql':
				$Sql .= ' LIMIT 0, 1';
				break;
			case 'pgsql':
				$Sql .= ' LIMIT 1 OFFSET 0';
				break;

			default:
				throw new Exception('Unhandled db-driver.');
		}

		$Logger->debug($Sql);
		$Logger->debug($ESObject->ESOBJECT_ID);

		$Row = $this->DB->getRow($Sql, DBMC_FETCHMODE_ASSOC);
		if ( Plattform::isError($Row) )
		{
			$Logger->debug('Error fetching row from db.');
			return false;
		}

		if ( $Row )
		{
			$Logger->debug('Fetched row from db -> instance exists.');
			$ESObject->setInstanceData($Row);
			return true;
		}

		$Logger->debug('Fetched NO row from db -> instance does NOT exists.');
		return false;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Interface::createInstance()
	 */
	public function createInstance(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Interface::process()
	 */
	public function process(
		$p_kind,
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		switch( strtolower($p_kind) )
		{
			case ESRender_Application_Interface::DISPLAY_MODE_DOWNLOAD:
				$Logger->debug('Calling Module::download()');
				return $this->download($RemoteApplication, $requestData	);
			break;

			case ESRender_Application_Interface::DISPLAY_MODE_INLINE:
				$Logger->debug('Calling Module::inline()');
				return $this->inline($RemoteApplication, $requestData	);
			break;

			case ESRender_Application_Interface::DISPLAY_MODE_WINDOW:
				$Logger->debug('Calling Module::display()');
				return $this->display($RemoteApplication, $requestData);
			break;

			default:
				throw new Exception('Unhandled display-kind "'.$p_kind.'".');
		}

		return true;
	}

	/**
	 * setCache
	 */
	public function setCache()
	{
		return true;
	}

	/**
	 *
	 * @param string $Name
	 *
	 * @return ESRender_Module_Base
	 */
	protected function setName($Name)
	{
		$this->Name = (string) $Name;
		return $this;
	}

	/**
	 *
	 * @return string
	 */
	public function getName()
	{
		return $this->Name;
	}

	/**
	 *
	 *
	 * @var ESRender_Application_Interface
	 */
	private $RenderApplication = null;

	/**
	 *
	 *
	 * @param ESRender_Application_Interface $RenderApplication
	 * @return ESRender_Module_Base
	 */
	protected function setRenderApplication(
		ESRender_Application_Interface $RenderApplication)
	{
		$this->RenderApplication = $RenderApplication;
		return $this;
	}

	/**
	 *
	 * @return ESRender_Application_Interface
	 */
	public function getRenderApplication()
	{
		return $this->RenderApplication;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Interface::getTimesOfUsage()
	 */
	public function getTimesOfUsage()
	{
		return 1;
	}

	/**
	 *
	 * @var Logger
	 */
	protected $Logger = null;

	/**
	 * Set the logger-instance to use
	 *
	 * @param Logger $Logger
	 */
	protected function setLogger(Logger $Logger)
	{
		$this->Logger = $Logger;
		return $this;
	}

	/**
	 *
	 * @return Logger
	 */
	protected function getLogger()
	{
		return $this->Logger;
	}

	/**
	 * Used to render templates.
	 *
	 * @var Phools_Template_Interface
	 */
	private $Template = null;

	/**
	 * Set the template-renderer to use
	 *
	 * @param Phools_Template_Interface $Template
	 */
	protected function setTemplate(
		Phools_Template_Interface $Template)
	{
		$this->Template = $Template;
		return $this;
	}

	/**
	 *
	 * @return Phools_Template_Interface
	 */
	protected function getTemplate()
	{
		return $this->Template;
	}

}
