<?php

/**
 *
 *
 *
 */
abstract class ESRender_Module_PhpVideoToolkit_Abstract
extends ESRender_Module_ContentNode_Abstract
{

	/**
	 * Format constants
	 *
	 * @var string
	 */
	const FORMAT_IPAD = 'FORMAT_IPAD';
	const FORMAT_FLASH = 'FORMAT_FLASH';

	/**
	 *
	 * @param string $format
	 *
	 * @throws Exception
	 *
	 * @return string
	 */
	abstract protected function getOutputFilename($format = self::FORMAT_FLASH);

	/**
	 *
	 * @param PHPVideoToolkit $toolkit to configure
	 * @param string $format
	 *
	 * @return PHPVideoToolkit the configured toolkit
	 */
	abstract protected function configPhpVideoToolkit(PHPVideoToolkit $toolkit, $format);

	/**
	 *
	 * @return string from this class' format-constants.
	 */
	protected function formatByUserAgent()
	{
		$format = self::FORMAT_FLASH;

		// return early
		if ( empty($_SERVER['HTTP_USER_AGENT']) )
		{
			return $format;
		}

		// iPhone
		if ( stripos($_SERVER['HTTP_USER_AGENT'], 'iPhone') ) {
			$format = self::FORMAT_IPAD;
		}

		// iPad
		if ( stripos($_SERVER['HTTP_USER_AGENT'], 'iPad') ) {
			$format = self::FORMAT_IPAD;
		}

		// Android
		if ( stripos($_SERVER['HTTP_USER_AGENT'], 'Android') ) {
			$format = self::FORMAT_IPAD;
		}

		return $format;
	}

	/**
	 *
	 * @throws Exception
	 *
	 * @return bool
	 */
	protected function convert($format)
	{
		$Logger = $this->getLogger();

		// !!! APPEND DIRECTORY_SEPARATOR to avoid confusion by phpVideoToolkit
		$tmp_dir = dirname($this->getCacheFileName()) . DIRECTORY_SEPARATOR;

		// init
		$toolkit = new PHPVideoToolkit($tmp_dir);
		$toolkit->on_error_die = FALSE;
		$toolkit->addCommand('-strict', 'experimental');

		// configure
		$toolkit = $this->configPhpVideoToolkit($toolkit, $format);

		// setup input-filename
		$input_filename = $this->getCacheFileName();
		if(!$toolkit->setInputFile($input_filename))
		{
			throw new Exception($toolkit->getLastError());
		}

		// setup output-filename
		$output_filename = $this->getOutputFilename($format);
		// !!! APPEND DIRECTORY_SEPARATOR to avoid confusion by phpVideoToolkit
		$output_dir = dirname($output_filename).DIRECTORY_SEPARATOR;
		if(!$toolkit->setOutput($output_dir, basename($output_filename), PHPVideoToolkit::OVERWRITE_EXISTING))
		{
			$logfile = $output_filename.'.log';
			$toolkit->moveLog($logfile);

			//  if there was an error then get it
			throw new Exception($toolkit->getLastError());
		}

			//  execute the ffmpeg command using multiple passes and log the calls and ffmpeg results

       global $CC_RENDER_PATH;
       
       $lock =$CC_RENDER_PATH.'.basename($output_filename).'lock';

	if (file_exists($lock)) {
            // nach dem alter der datei schauen 
				if ( $Logger ) {
				$Logger->debug( 'found old lock file:'.$lock );
				}
				} else {
				 if (!$handle = fopen($lock, "a")) {
				 	        fclose($handle);
						if ( $Logger ) {
						$Logger->debug( "can't create lock file : $lock" );
						}
				         exit;
				    }
			if ( $Logger ) {
					$Logger->debug( "start video converting : $output_filename" );
					}
				fclose($handle);

$result = false;
$result = $toolkit->execute(true, false);
		

    if ($result){ unlink($lock);
					if ( $Logger ) {
					$Logger->debug( "delete lock file : $lock" );
						}
				}
		}



//		$result = $toolkit->execute(true, true);



		if(PHPVideoToolkit::RESULT_OK !== $result)
		{
			if ( $Logger ) {
				$Logger->debug( $toolkit->getLastCommand() );
			}

			$logfile = $output_filename.'.log';
			$toolkit->moveLog($logfile);

			//  if there was an error then get it
			error_log(print_r($toolkit->getLastError(), true));

			return false;
		}

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_ContentNode_Abstract::createInstance()
	 */
	final public function createInstance(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		if ( ! parent::createInstance($RemoteApplication, $requestData) )
		{
			return false;
		}

		$format = $this->formatByUserAgent();
		$this->convert($format);

		return true;
	}

	/**
	 * Checking for HTTP-header "User-Agent" allows conversion to a supported
	 * audio/video-format "on-the-fly".
	 *
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::process()
	 */
	public function process(
		$p_kind,
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$format = $this->formatByUserAgent();

		$output_filename = $this->getOutputFilename($format);
		if ( ! file_exists($output_filename) )
		{
			$this->convert($format);
		}

		return parent::process($p_kind, $RemoteApplication, $requestData);
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::getTimesOfUsage()
	 */
	public function getTimesOfUsage()
	{
		return PHP_INT_MAX;
	}

}
