<?php
/*
* $McLicense$
*
* $Id$
*
*/

require_once("MDB2.php"); // from PEAR

define('DBMC_FETCHMODE_DEFAULT', MDB2_FETCHMODE_DEFAULT);
define('DBMC_FETCHMODE_ORDERED', MDB2_FETCHMODE_ORDERED);
define('DBMC_FETCHMODE_ASSOC',   MDB2_FETCHMODE_ASSOC);
define('DBMC_FETCHMODE_OBJECT',  MDB2_FETCHMODE_OBJECT);
define('DBMC_FETCHMODE_FLIPPED', MDB2_FETCHMODE_FLIPPED);

define("DB_DEBUG_MESSAGES",	true);

/**
 * class DBMC
 *
 * fakes original database wrapper for working without PEAR and all
 */
class DBMC
{
	/**
	 * connection parameters
	 */
	private $drvr = null;
	private $host = null;
	private $port = null;
	private $user = null;
	private $pass = null;
	private $name = null;


	/**
	 * database connection
	 */
	private $DB = null;

	/**
	 * constructor
	 */
	public function __construct($drvr, $host, $port, $user, $pass, $name = null)
	{
		$this->drvr = $drvr;
		$this->host = $host;
		$this->port = $port;
		$this->user = $user;
		$this->pass = $pass;
		$this->name = $name;

  	  $ConnectionResource = MDB2::connect($this->getDSN(), false);
		
		if (PEAR::isError($ConnectionResource)) {
			throw new Exception($ConnectionResource->getUserInfo());
    }
		
		
		DBMC::configureConnection($ConnectionResource);

		$this->DB = $ConnectionResource;
	}



	/**
	 * @return getDSN
	 */
	private function getDSN()
	{
		/**
		 * data source name (see PEAR DB API)
		 * "{driver}://{$user}:{$pass}@tcp+{$host}:{$port}/{$name}";
		 */
		$DSN = "{$this->drvr}://{$this->user}:{$this->pass}@tcp+{$this->host}:{$this->port}/{$this->name}";

		return $DSN;
	}



	/**
	 * @return getDefaultPort
	 */
	public static function getDefaultPort($driver=null)
	{
#		if ($driver == 'mysql') {
			return '3306';
#		}
	}



	/**
	 * configureConnection
	 *
	 * @param $objDb database connection, instance of class MDB2
	 *
	 * @return bool
	 */
	public static final function configureConnection($objDb)
	{
		$objDb->setOption('portability', MDB2_PORTABILITY_ALL ^ MDB2_PORTABILITY_EMPTY_TO_NULL ^ MDB2_PORTABILITY_FIX_CASE);
		$objDb->query('set NAMES utf8');
		$objDb->query("set sql_mode = 'NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION'");
	}



	/**
	 * setOption
	 *
	 * @param string $name
	 * @param mixed $value
	 *
	 * @return ???
	 */
	function setOption($name, $value)
	{
		return $this->DB->setOption($name, $value);
	}



	/**
	 * isError
	 *
	 * @param ??? $object
	 *
	 * @return bool
	 */
	public static function isError(&$object)
	{
		return MDB2::isError($object);
	}



	/**
	 *
	 * @param ??? $object
	 * @return ???
	 */
	private static function checkResult($object)
	{
		if ( ! MDB2::isError($object) )
		{
			return;
		}

		self::log($object);

		$info = $object->getUserInfo();

		$loweredInfo = strtolower($info);
		if (strpos($loweredInfo, 'error 28 from storage engine') !== false)
		{
			throw new Exception('database error: no space left on device');
		}

		if (strpos($loweredInfo, 'client ran out of memory') !== false)
		{
			throw new Exception('database error: client ran out of memory');
		}

		throw new Exception("database error: {$info}");
	}



	/**
	 * from MDB2::mixed &query( string $query, [mixed $types = null], [mixed $result_class = true], [mixed $result_wrap_class = false])
	 *
	 * @param ??? $query
	 * @param ??? $types
	 * @param bool $result_class
	 * @param bool $result_wrap_class
	 *
	 * @return ???
	 */
	function &query($query, $types = null, $result_class = true, $result_wrap_class = false)
	{
		$result = $this->DB->query($query, $types, $result_class, $result_wrap_class);
		$this->checkResult($result);

		return $result;
	}



	/**
	 * method is deprecated (from deprecated package Pear::DB)
	 *
	 * @deprecated from deprecated package Pear::DB
	 *
	 * @param ??? $query
	 *
	 * @return ???
	 */
	function getOne($query)
	{
		$result = $this->DB->queryOne($query);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::mixed queryOne( string $query, [string $type = null], [int $colnum = 0])
	 *
	 * @param ??? $query
	 * @param ??? $type
	 * @param ??? $column
	 *
	 * @return
	 */
	function queryOne($query, $type = null, $column = 0)
	{
		$result = $this->DB->queryOne($query, $type, $colnum);
		self::checkResult($result);

		return $result;
	}



	/**
	 * @deprecated method is deprecated (from deprecated package Pear::DB)
	 *
	 * @param ??? $query
	 *
	 * @return ???
	 */
	function getCol($query)
	{
		$result = $this->DB->queryCol($query);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::mixed queryCol( string $query, [string $type = null], [int $colnum = 0])
	 *
	 * @param ??? $query
	 * @param ??? $type
	 * @param int $column
	 *
	 * @return ???
	 */
	function queryCol($query, $type = null, $column = 0)
	{
		$result = $this->DB->queryCol($query, $type, $column);
		self::checkResult($result);

		return $result;
	}



	/**
	 * @deprecated method is deprecated (from deprecated package Pear::DB)
	 *
	 * @param ??? $query
	 * @param ???
	 *
	 * @return ???
	 */
	function getRow($query, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$result = $this->DB->queryRow($query, null, $fetchmode);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::mixed queryRow( string $query, [array $types = null], [int $fetchmode = MDB2_FETCHMODE_DEFAULT])
	 *
	 * @param ??? $query
	 * @param ??? $types
	 * @param ??? $fetchmode
	 *
	 * @return ???
	 */
	function queryRow($query, $types = null, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$result = $this->DB->queryRow($query, $types, $fetchmode);
		self::checkResult($result);

		return $result;
	}



	/**
	 * @deprecated method is deprecated (from deprecated package Pear::DB)
	 *
	 * @param ??? $query
	 * @param ??? $fetchmode
	 *
	 * @return ???
	 */
	function getAll($query, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$result = $this->DB->queryAll($query, null, $fetchmode);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::mixed queryAll( string $query, [array $types = null], [int $fetchmode = MDB2_FETCHMODE_DEFAULT], [bool $rekey = false], [bool $force_array = false], [bool $group = false])
	 *
	 * @param ??? $query
	 * @param ??? $types
	 * @param ??? $fetchmode
	 * @param ??? $rekey
	 * @param ??? $force_array
	 * @param ??? $group
	 *
	 *  @return ???
	 */
	function queryAll($query, $types = null, $fetchmode = DBMC_FETCHMODE_DEFAULT, $rekey = false, $force_array = false, $group = false)
	{
		$result = $this->DB->queryAll($query, $types, $fetchmode, $rekey, $force_array, $group);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::mixed lastInsertID( [string $table = null], [string $field = null])
	 *
	 * @return
	 */
	function getLastInsertId()
	{
		if (func_num_args() > 0)
		{
			// display notice because arguments table and field are not supported here
		}

		$result = $this->DB->lastInsertID();
		self::checkResult($result);

		return ( intval($result) );
	}



	/**
	 * from MDB2::mixed &prepare( string $query, [mixed $types = null], [mixed $result_types = null], [mixed $lobs = array()])
	 *
	 * @param ??? $query
	 *
	 * @return ???
	 */
	function &prepare($query)
	{
		$result = $this->DB->prepare($query);
		self::checkResult($result);

		return $result;
	}



	/**
	 *
	 * @param $prepare
	 * @param $values
	 *
	 * @return ???
	 */
	function execute($objPrepare, $arrValues)
	{
		$result = $objPrepare->execute($arrValues);
		self::checkResult($result);

		return $result;
	}



	/**
	 * from MDB2::string quote( string $value, [string $type = null], [bool $quote = true], [bool $escape_wildcards = false])
	 *
	 * @param ??? $value
	 * @param ??? $type
	 * @param ??? $quote
	 * @param ??? $escape_wildcards
	 *
	 * @return ???
	 */
	public function quote($mixedValue, $type = null, $quote = true, $escape_wildcards = false)
	{
		if ( ! is_array($mixedValue) )
		{
			return $this->DB->quote($mixedValue, $type, $quote, $escape_wildcards);
		}

		// indicate if we're looping over the first item
		$arrQuotedValue = array();

		while( $value = array_shift($mixedValue) )
		{
			$arrQuotedValue[] = $this->quote($value, $type, $quote, $escape_wildcards);
		}

		$quotedValue = implode(', ', $arrQuotedValue);

		return $quotedValue;
	}



	/**
	 * log method for database actions
	 *
	 * @param string $txt
	 *
	 * @return ???
	 */
	private static function log($errorObj)
	{
		$info = $errorObj->getUserInfo();
#		$query = $object->getLastQuery();

		$logText = 'Error: '.$info;

		$date = date('Ymd');
		$logFileError = INST_PATH_LOGS . "db_error.{$date}.log";

		error_log($logText, 3, $logFileError);
	}



	/**
	 * createDb
	 */
	function createDb($name)
	{
		$this->query("create database {$name} DEFAULT CHARACTER SET utf8");
		return true;
	}



	/**
	 * dbExists
	 */
	function dbExists($name)
	{
		$res = $this->query("show databases like \"{$name}\"");
		if ($res->numRows() == 0)
		{
			return false;
		}

		// database already exists
		return true;
	}



	/**
	 * selectDb
	 *
	 * selects the given database and reminds its name in member $dbname
	 */
	function selectDb($name)
	{
		$this->name = $name;

		$ConnectionResource = MDB2::connect($this->getDSN(), false);

		DBMC::configureConnection($ConnectionResource);

		$this->DB = $ConnectionResource;

		return true;
	}



	/**
 * getVersion
	 *
	 * selects the version of database server
	 */
	function getVersion()
	{
		$res = $this->getOne('select version()');
		return $res;
	}



	/**
	 * writeSysConfig
	 *
	 * writes a value into system config table
	 * returns true or false
	 */
	function writeSysConfig($name, $value)
	{
		$tab   = 'MC_CONFIG';
		$name  = $this->quote($name);
		$value = $this->quote($value);

		$upd = "update {$tab} set {$tab}_VALUE={$value} where {$tab}_NAME={$name}";
		$res = $this->query($upd);

		return true;
	}



	/**
	 * error
	 *
	 * creates an error message from given parameters
	 */
	function error($param, $msg = null)
	{
		include_once(INST_PATH_LIB.'ErrorHandler.php');
		mc_ErrorHandler::handleError(null, $param, $msg);
	}

}

