<?php
/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

include_once ('../../conf.inc.php');


/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class mod_qti21
extends ESRender_Module_ContentNode_Abstract
{

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::display()
	 */
	final protected function display(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		require_once('config.php');

		$Logger = $this->getLogger();

		try {
			$m_mimeType = $this->_ESOBJECT->getMimeType();
			$m_path = $this->_ESOBJECT->getFilePath();
			$m_name = $this->_ESOBJECT->getTitle();
			$m_objectID = $this->_ESOBJECT->getObjectID();

			$SoapClientParams = array();
			if ( defined('USE_HTTP_PROXY') && USE_HTTP_PROXY )
			{
			        $SoapClientParams = array(
			                'proxy_host' => HTTP_PROXY_HOST,
			                'proxy_port' => HTTP_PROXY_PORT,
			                'proxy_login' => HTTP_PROXY_USER,
			                'proxy_password' => HTTP_PROXY_PASS,
			        );
			}

			$client = new SoapClient(cfg_onyx_service, $SoapClientParams);
			$file = $m_path;

			$fh = fopen($file, 'r+');
			if ( ! $fh )
			{
				$Logger->error('Error opening file "'.$file.'".');
				return false;
			}

			$contents = file_get_contents($file, false);
			if ( ! $contents )
			{
				$Logger->error('Error reading file-contents "'.$file.'".');
				return false;
			}

			$wrappedParams = new stdClass();
			$wrappedParams->uniqueId = $requestData["rep_id"]."-".$requestData["app_id"]."-".$requestData["course_id"]."-".$requestData["user_id"]."-".$requestData["tracking_id"]."-".$requestData['object_id'];
			$wrappedParams->contentPackage = $contents;
			$wrappedParams->language = 'de';
			$wrappedParams->instructions = '<html><body><h1>O N Y X</h1></body></html>';
			$wrappedParams->tempalteId  = 'onyxdefault';
			// @todo move to config
			$wrappedParams->serviceName =  'esrender';
			$wrappedParams->allowShowSolution = 'true';

			$result = $client->run(
				$wrappedParams->uniqueId,
				$wrappedParams->contentPackage,
				$wrappedParams->language,
				$wrappedParams->instructions,
				$wrappedParams->tempalteId,
				$wrappedParams->serviceName,
				$wrappedParams->allowShowSolution);

			$oru = cfg_onyx_runurl."?id=".urlencode($wrappedParams->uniqueId);
			$oru .= '&' . session_name() . '=' . session_id();

			if ( $Logger ) {
				$Logger->debug('Redirecting to url: "'.$oru.'"');
			}

			$Template = $this->getTemplate();
			echo $Template->render('/module/qti21/display', array(
				'oru' => $oru,
			));

		} catch(Exception $e){
			echo "<pre>";
			throw new SoapFault("Server", " error:".$e);
		}

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::createInstance()
	 */
	final public function createInstance(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		if ( ! parent::createInstance($RemoteApplication, $requestData) )
		{
			return false;
		}

	  	$qti_zip_path = $this->render_path.DIRECTORY_SEPARATOR.$this->filename;

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::process()
	 */
	final public function process(
		$p_kind,
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$m_mimeType = $this->_ESOBJECT->getMimeType();
		$m_path = $this->_ESOBJECT->getPath();
		$m_name = $this->_ESOBJECT->getTitle();
		$obj_module = $this->_ESOBJECT->getModule();
		$p_kind = $obj_module->getConf('defaultdisplay', $p_kind);

		switch($p_kind)
		{
			case ESRender_Application_Interface::DISPLAY_MODE_DOWNLOAD:
				header('Content-type: '.$m_mimeType);
				header('Content-Disposition: attachment; filename="'.$m_name.'"');
				readfile($m_path);
			break;

			case ESRender_Application_Interface::DISPLAY_MODE_WINDOW:
				return $this->display($RemoteApplication, $requestData);
			break;

			default:
				return false;
		}

		return true;
	}

}

